# -*- coding: utf-8 -*-
import logging
import sys
import click
import click_log
import boto3
from src.core.services.aws.ec2 import Ec2
from src.utils.format import grid

logger = logging.getLogger()
logging.getLogger("boto3").setLevel(logging.CRITICAL)
logging.getLogger("botocore").setLevel(logging.CRITICAL)
logging.getLogger("s3transfer").setLevel(logging.CRITICAL)
logging.getLogger("urllib3").setLevel(logging.CRITICAL)
click_log.basic_config(logger)
logger.setLevel(logging.INFO)
click_log.ColorFormatter.colors["info"] = dict(fg="green")
click_log.ColorFormatter.colors["warning"] = dict(fg="yellow")
click_log.ColorFormatter.colors["error"] = dict(fg="red")


@click.group()
def cli():
    pass


@cli.command()
@click.option(
    "--provider",
    prompt="Cloud Provider",
    default="aws",
    help="Get your Cloud Provider.",
)
@click.option(
    "--service", prompt="Service name", default="ec2", help="Get Service Name."
)
@click.option(
    "--region", prompt="Provider region", default="us-east-1", help="Get Region Name."
)
def analyze(provider, service, region):
    logger.info("Launch analyze for %s on %s ..." % (service, provider))
    if provider == "aws":
        if service == "ec2":
            ec2 = Ec2(boto3, region).analyze()
            grid(ec2, service, provider)
        else:
            logger.info(str(service) + " don't allow!")
            sys.exit(1)
    else:
        logger.info(str(provider) + " don't allow!")
        sys.exit(1)


@cli.command()
@click.option(
    "--provider",
    prompt="Cloud Provider",
    default="aws",
    help="Get your Cloud Provider.",
)
@click.option(
    "--service", prompt="Service name", default="ec2", help="Get Service Name."
)
@click.option(
    "--region", prompt="Provider region", default="us-east-1", help="Get Region Name."
)
def report(provider, service, region):
    click.echo("Launch report for %s on %s!" % (service, provider))


if __name__ == "__main__":
    cli()



# -*- coding: utf-8 -*-
import logging
import boto3

logger = logging.getLogger(__name__)


def account_id():
    response = boto3.client("sts").get_caller_identity().get("Account")
    logger.info("Account ID: " + response)
    return response



# -*- coding: utf-8 -*-
from setuptools import find_packages, setup

# from package import Package

setup(
    name="CostReduce",
    description="An essential toolset that eases server administration",
    version="0.0.1",
    author="Maxence Maireaux",
    author_email="maxence@maireaux.fr",
    url="https://www.costreduce.io",
    license="MIT",
    python_requires=">=3.6,<4.0",
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    entry_points="""
          [console_scripts]
          costreduce = src.cli.main:cli
      """,
)



# -*- coding: utf-8 -*-
import logging

logger = logging.getLogger(__name__)


class Ec2:
    def __init__(self, sts, region):
        self.region = region
        self.client_ec2 = sts.client("ec2", region_name=region)
        self.client_elbv2 = sts.client("elbv2", region_name=region)

    def analyze(self):
        analyze = list()
        # EIP
        logger.info("Check for EIP")
        analyze.append(self._free_eip())
        # ALB
        logger.info("Check for ALB")
        analyze.append(self._alb_listener_one_rule())

        logger.debug("Result of analyze : " + str(analyze))
        return analyze

    def _free_eip(self):
        addresses_dict = self.client_ec2.describe_addresses()
        response = list()
        for eip_dict in addresses_dict["Addresses"]:
            if "InstanceId" not in eip_dict:
                data = "Remove EIPs when not in use : " + str(eip_dict["AllocationId"])
                logger.debug(data)
                response.append(data)
        return response

    def _alb_listener_one_rule(self):
        response = list()
        alb_listener_arn = self.__alb_listener_arn()
        for listenerArn in alb_listener_arn:
            rules_dict = self.client_elbv2.describe_rules(ListenerArn=listenerArn)
            for rule in rules_dict["Rules"]:
                count = 0
                for i in rule["Conditions"]:
                    if rule["Priority"] != "default":
                        # print(rule)
                        count += 1
                if count < 2:
                    listener_to_elb = self.client_elbv2.describe_listeners(
                        ListenerArns=[listenerArn]
                    )
                    elb_arn = listener_to_elb["Listeners"][0]["LoadBalancerArn"]
                    data = (
                        "Fusion this ALB "
                        + str(elb_arn)
                        + " because one ALB for one rules is so expensive."
                    )
                    if data not in response:
                        logger.debug(data)
                        response.append(data)
        return response

    def __alb_listener_arn(self):
        response = list()
        alb_arn = self.get_elb_arn()
        for alb in alb_arn:
            listeners_dict = self.client_elbv2.describe_listeners(LoadBalancerArn=alb)
            for listener in listeners_dict["Listeners"]:
                response.append(listener["ListenerArn"])
        return response

    def get_elb_arn(self):
        alb_dict = self.client_elbv2.describe_load_balancers()
        response = list()
        for alb in alb_dict["LoadBalancers"]:
            response.append(alb["LoadBalancerArn"])
        return response
